@@ -1,39 +1,19 @@
 """This component provides basic support for Foscam IP cameras."""
+from __future__ import annotations
+
 import asyncio
 
 from libpyfoscam import FoscamCamera
 import voluptuous as vol
 
-from homeassistant.components.camera import PLATFORM_SCHEMA, SUPPORT_STREAM, Camera
-from homeassistant.config_entries import SOURCE_IMPORT
-from homeassistant.const import (
-    CONF_HOST,
-    CONF_NAME,
-    CONF_PASSWORD,
-    CONF_PORT,
-    CONF_USERNAME,
-)
+from homeassistant.components.camera import Camera, CameraEntityFeature
+from homeassistant.config_entries import ConfigEntry
+from homeassistant.const import CONF_HOST, CONF_PASSWORD, CONF_PORT, CONF_USERNAME
+from homeassistant.core import HomeAssistant
 from homeassistant.helpers import config_validation as cv, entity_platform
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
-from .const import (
-    CONF_RTSP_PORT,
-    CONF_STREAM,
-    DOMAIN,
-    LOGGER,
-    SERVICE_PTZ,
-    SERVICE_PTZ_PRESET,
-)
-
-PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend(
-    {
-        vol.Required("ip"): cv.string,
-        vol.Required(CONF_PASSWORD): cv.string,
-        vol.Required(CONF_USERNAME): cv.string,
-        vol.Optional(CONF_NAME, default="Foscam Camera"): cv.string,
-        vol.Optional(CONF_PORT, default=88): cv.port,
-        vol.Optional(CONF_RTSP_PORT): cv.port,
-    }
-)
+from .const import CONF_RTSP_PORT, CONF_STREAM, LOGGER, SERVICE_PTZ, SERVICE_PTZ_PRESET
 
 DIR_UP = "up"
 DIR_DOWN = "down"
@@ -65,32 +45,13 @@ ATTR_PRESET_NAME = "preset_name"
 PTZ_GOTO_PRESET_COMMAND = "ptz_goto_preset"
 
 
-async def async_setup_platform(hass, config, async_add_entities, discovery_info=None):
-    """Set up a Foscam IP Camera."""
-    LOGGER.warning(
-        "Loading foscam via platform config is deprecated, it will be automatically imported. Please remove it afterwards."
-    )
-
-    config_new = {
-        CONF_NAME: config[CONF_NAME],
-        CONF_HOST: config["ip"],
-        CONF_PORT: config[CONF_PORT],
-        CONF_USERNAME: config[CONF_USERNAME],
-        CONF_PASSWORD: config[CONF_PASSWORD],
-        CONF_STREAM: "Main",
-        CONF_RTSP_PORT: config.get(CONF_RTSP_PORT, 554),
-    }
-
-    hass.async_create_task(
-        hass.config_entries.flow.async_init(
-            DOMAIN, context={"source": SOURCE_IMPORT}, data=config_new
-        )
-    )
-
-
-async def async_setup_entry(hass, config_entry, async_add_entities):
+async def async_setup_entry(
+    hass: HomeAssistant,
+    config_entry: ConfigEntry,
+    async_add_entities: AddEntitiesCallback,
+) -> None:
     """Add a Foscam IP camera from a config entry."""
-    platform = entity_platform.current_platform.get()
+    platform = entity_platform.async_get_current_platform()
     platform.async_register_entity_service(
         SERVICE_PTZ,
         {
@@ -133,20 +94,21 @@ async def async_setup_entry(hass, config_entry, async_add_entities):
 class HassFoscamCamera(Camera):
     """An implementation of a Foscam IP camera."""
 
-    def __init__(self, camera, config_entry):
+    def __init__(self, camera: FoscamCamera, config_entry: ConfigEntry) -> None:
         """Initialize a Foscam camera."""
         super().__init__()
 
         self._foscam_session = camera
-        self._name = config_entry.title
+        self._attr_name = config_entry.title
         self._username = config_entry.data[CONF_USERNAME]
         self._password = config_entry.data[CONF_PASSWORD]
         self._stream = config_entry.data[CONF_STREAM]
-        self._unique_id = config_entry.entry_id
+        self._attr_unique_id = config_entry.entry_id
         self._rtsp_port = config_entry.data[CONF_RTSP_PORT]
-        self._motion_status = False
+        if self._rtsp_port:
+            self._attr_supported_features = CameraEntityFeature.STREAM
 
-    async def async_added_to_hass(self):
+    async def async_added_to_hass(self) -> None:
         """Handle entity addition to hass."""
         # Get motion detection status
         ret, response = await self.hass.async_add_executor_job(
@@ -156,23 +118,20 @@ class HassFoscamCamera(Camera):
         if ret == -3:
             LOGGER.info(
                 "Can't get motion detection status, camera %s configured with non-admin user",
-                self._name,
+                self.name,
             )
 
         elif ret != 0:
             LOGGER.error(
-                "Error getting motion detection status of %s: %s", self._name, ret
+                "Error getting motion detection status of %s: %s", self.name, ret
             )
 
         else:
-            self._motion_status = response == 1
+            self._attr_motion_detection_enabled = response == 1
 
-    @property
-    def unique_id(self):
-        """Return the entity unique ID."""
-        return self._unique_id
-
-    def camera_image(self):
+    def camera_image(
+        self, width: int | None = None, height: int | None = None
+    ) -> bytes | None:
         """Return a still image response from the camera."""
         # Send the request to snap a picture and return raw jpg data
         # Handle exception if host is not reachable or url failed
@@ -182,27 +141,14 @@ class HassFoscamCamera(Camera):
 
         return response
 
-    @property
-    def supported_features(self):
-        """Return supported features."""
-        if self._rtsp_port:
-            return SUPPORT_STREAM
-
-        return None
-
-    async def stream_source(self):
+    async def stream_source(self) -> str | None:
         """Return the stream source."""
         if self._rtsp_port:
             return f"rtsp://{self._username}:{self._password}@{self._foscam_session.host}:{self._rtsp_port}/video{self._stream}"
 
         return None
 
-    @property
-    def motion_detection_enabled(self):
-        """Camera Motion Detection Status."""
-        return self._motion_status
-
-    def enable_motion_detection(self):
+    def enable_motion_detection(self) -> None:
         """Enable motion detection in camera."""
         try:
             ret = self._foscam_session.enable_motion_detection()
@@ -211,18 +157,18 @@ class HassFoscamCamera(Camera):
                 if ret == -3:
                     LOGGER.info(
                         "Can't set motion detection status, camera %s configured with non-admin user",
-                        self._name,
+                        self.name,
                     )
                 return
 
-            self._motion_status = True
+            self._attr_motion_detection_enabled = True
         except TypeError:
             LOGGER.debug(
                 "Failed enabling motion detection on '%s'. Is it supported by the device?",
-                self._name,
+                self.name,
             )
 
-    def disable_motion_detection(self):
+    def disable_motion_detection(self) -> None:
         """Disable motion detection."""
         try:
             ret = self._foscam_session.disable_motion_detection()
@@ -231,27 +177,27 @@ class HassFoscamCamera(Camera):
                 if ret == -3:
                     LOGGER.info(
                         "Can't set motion detection status, camera %s configured with non-admin user",
-                        self._name,
+                        self.name,
                     )
                 return
 
-            self._motion_status = False
+            self._attr_motion_detection_enabled = False
         except TypeError:
             LOGGER.debug(
                 "Failed disabling motion detection on '%s'. Is it supported by the device?",
-                self._name,
+                self.name,
             )
 
     async def async_perform_ptz(self, movement, travel_time):
         """Perform a PTZ action on the camera."""
-        LOGGER.debug("PTZ action '%s' on %s", movement, self._name)
+        LOGGER.debug("PTZ action '%s' on %s", movement, self.name)
 
         movement_function = getattr(self._foscam_session, MOVEMENT_ATTRS[movement])
 
         ret, _ = await self.hass.async_add_executor_job(movement_function)
 
         if ret != 0:
-            LOGGER.error("Error moving %s '%s': %s", movement, self._name, ret)
+            LOGGER.error("Error moving %s '%s': %s", movement, self.name, ret)
             return
 
         await asyncio.sleep(travel_time)
@@ -261,12 +207,12 @@ class HassFoscamCamera(Camera):
         )
 
         if ret != 0:
-            LOGGER.error("Error stopping movement on '%s': %s", self._name, ret)
+            LOGGER.error("Error stopping movement on '%s': %s", self.name, ret)
             return
 
     async def async_perform_ptz_preset(self, preset_name):
         """Perform a PTZ preset action on the camera."""
-        LOGGER.debug("PTZ preset '%s' on %s", preset_name, self._name)
+        LOGGER.debug("PTZ preset '%s' on %s", preset_name, self.name)
 
         preset_function = getattr(self._foscam_session, PTZ_GOTO_PRESET_COMMAND)
 
@@ -274,11 +220,6 @@ class HassFoscamCamera(Camera):
 
         if ret != 0:
             LOGGER.error(
-                "Error moving to preset %s on '%s': %s", preset_name, self._name, ret
+                "Error moving to preset %s on '%s': %s", preset_name, self.name, ret
             )
             return
-
-    @property
-    def name(self):
-        """Return the name of this camera."""
-        return self._name
