@@ -8,7 +8,7 @@ from typing import Final
 
 from pyfritzhome.fritzhomedevice import FritzhomeDevice
 
-from homeassistant.components.climate.const import PRESET_COMFORT, PRESET_ECO
+from homeassistant.components.climate import PRESET_COMFORT, PRESET_ECO
 from homeassistant.components.sensor import (
     SensorDeviceClass,
     SensorEntity,
@@ -30,7 +30,7 @@ from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.typing import StateType
 from homeassistant.util.dt import utc_from_timestamp
 
-from . import FritzBoxEntity
+from . import FritzBoxDeviceEntity
 from .const import CONF_COORDINATOR, DOMAIN as FRITZBOX_DOMAIN
 from .model import FritzEntityDescriptionMixinBase
 
@@ -49,6 +49,56 @@ class FritzSensorEntityDescription(
     """Description for Fritz!Smarthome sensor entities."""
 
 
+def suitable_eco_temperature(device: FritzhomeDevice) -> bool:
+    """Check suitablity for eco temperature sensor."""
+    return device.has_thermostat and device.eco_temperature is not None
+
+
+def suitable_comfort_temperature(device: FritzhomeDevice) -> bool:
+    """Check suitablity for comfort temperature sensor."""
+    return device.has_thermostat and device.comfort_temperature is not None
+
+
+def suitable_nextchange_temperature(device: FritzhomeDevice) -> bool:
+    """Check suitablity for next scheduled temperature sensor."""
+    return device.has_thermostat and device.nextchange_temperature is not None
+
+
+def suitable_nextchange_time(device: FritzhomeDevice) -> bool:
+    """Check suitablity for next scheduled changed time sensor."""
+    return device.has_thermostat and device.nextchange_endperiod is not None
+
+
+def suitable_temperature(device: FritzhomeDevice) -> bool:
+    """Check suitablity for temperature sensor."""
+    return device.has_temperature_sensor and not device.has_thermostat
+
+
+def value_electric_current(device: FritzhomeDevice) -> float:
+    """Return native value for electric current sensor."""
+    if (
+        isinstance(device.power, int)
+        and isinstance(device.voltage, int)
+        and device.voltage > 0
+    ):
+        return round(device.power / device.voltage, 3)
+    return 0.0
+
+
+def value_nextchange_preset(device: FritzhomeDevice) -> str:
+    """Return native value for next scheduled preset sensor."""
+    if device.nextchange_temperature == device.eco_temperature:
+        return PRESET_ECO
+    return PRESET_COMFORT
+
+
+def value_scheduled_preset(device: FritzhomeDevice) -> str:
+    """Return native value for current scheduled preset sensor."""
+    if device.nextchange_temperature == device.eco_temperature:
+        return PRESET_COMFORT
+    return PRESET_ECO
+
+
 SENSOR_TYPES: Final[tuple[FritzSensorEntityDescription, ...]] = (
     FritzSensorEntityDescription(
         key="temperature",
@@ -57,9 +107,7 @@ SENSOR_TYPES: Final[tuple[FritzSensorEntityDescription, ...]] = (
         device_class=SensorDeviceClass.TEMPERATURE,
         state_class=SensorStateClass.MEASUREMENT,
         entity_category=EntityCategory.DIAGNOSTIC,
-        suitable=lambda device: (
-            device.has_temperature_sensor and not device.has_thermostat
-        ),
+        suitable=suitable_temperature,
         native_value=lambda device: device.temperature,  # type: ignore[no-any-return]
     ),
     FritzSensorEntityDescription(
@@ -87,7 +135,7 @@ SENSOR_TYPES: Final[tuple[FritzSensorEntityDescription, ...]] = (
         device_class=SensorDeviceClass.POWER,
         state_class=SensorStateClass.MEASUREMENT,
         suitable=lambda device: device.has_powermeter,  # type: ignore[no-any-return]
-        native_value=lambda device: device.power / 1000 if device.power else 0.0,
+        native_value=lambda device: round((device.power or 0.0) / 1000, 3),
     ),
     FritzSensorEntityDescription(
         key="voltage",
@@ -96,9 +144,7 @@ SENSOR_TYPES: Final[tuple[FritzSensorEntityDescription, ...]] = (
         device_class=SensorDeviceClass.VOLTAGE,
         state_class=SensorStateClass.MEASUREMENT,
         suitable=lambda device: device.has_powermeter,  # type: ignore[no-any-return]
-        native_value=lambda device: device.voltage
-        if getattr(device, "voltage", None)
-        else 0.0,
+        native_value=lambda device: round((device.voltage or 0.0) / 1000, 2),
     ),
     FritzSensorEntityDescription(
         key="electric_current",
@@ -107,9 +153,7 @@ SENSOR_TYPES: Final[tuple[FritzSensorEntityDescription, ...]] = (
         device_class=SensorDeviceClass.CURRENT,
         state_class=SensorStateClass.MEASUREMENT,
         suitable=lambda device: device.has_powermeter,  # type: ignore[no-any-return]
-        native_value=lambda device: device.power / device.voltage / 1000
-        if device.power and getattr(device, "voltage", None)
-        else 0.0,
+        native_value=value_electric_current,
     ),
     FritzSensorEntityDescription(
         key="total_energy",
@@ -118,7 +162,7 @@ SENSOR_TYPES: Final[tuple[FritzSensorEntityDescription, ...]] = (
         device_class=SensorDeviceClass.ENERGY,
         state_class=SensorStateClass.TOTAL_INCREASING,
         suitable=lambda device: device.has_powermeter,  # type: ignore[no-any-return]
-        native_value=lambda device: device.energy / 1000 if device.energy else 0.0,
+        native_value=lambda device: (device.energy or 0.0) / 1000,
     ),
     # Thermostat Sensors
     FritzSensorEntityDescription(
@@ -126,8 +170,7 @@ SENSOR_TYPES: Final[tuple[FritzSensorEntityDescription, ...]] = (
         name="Comfort Temperature",
         native_unit_of_measurement=TEMP_CELSIUS,
         device_class=SensorDeviceClass.TEMPERATURE,
-        suitable=lambda device: device.has_thermostat
-        and device.comfort_temperature is not None,
+        suitable=suitable_comfort_temperature,
         native_value=lambda device: device.comfort_temperature,  # type: ignore[no-any-return]
     ),
     FritzSensorEntityDescription(
@@ -135,8 +178,7 @@ SENSOR_TYPES: Final[tuple[FritzSensorEntityDescription, ...]] = (
         name="Eco Temperature",
         native_unit_of_measurement=TEMP_CELSIUS,
         device_class=SensorDeviceClass.TEMPERATURE,
-        suitable=lambda device: device.has_thermostat
-        and device.eco_temperature is not None,
+        suitable=suitable_eco_temperature,
         native_value=lambda device: device.eco_temperature,  # type: ignore[no-any-return]
     ),
     FritzSensorEntityDescription(
@@ -144,35 +186,27 @@ SENSOR_TYPES: Final[tuple[FritzSensorEntityDescription, ...]] = (
         name="Next Scheduled Temperature",
         native_unit_of_measurement=TEMP_CELSIUS,
         device_class=SensorDeviceClass.TEMPERATURE,
-        suitable=lambda device: device.has_thermostat
-        and device.nextchange_temperature is not None,
+        suitable=suitable_nextchange_temperature,
         native_value=lambda device: device.nextchange_temperature,  # type: ignore[no-any-return]
     ),
     FritzSensorEntityDescription(
         key="nextchange_time",
         name="Next Scheduled Change Time",
         device_class=SensorDeviceClass.TIMESTAMP,
-        suitable=lambda device: device.has_thermostat
-        and device.nextchange_endperiod is not None,
+        suitable=suitable_nextchange_time,
         native_value=lambda device: utc_from_timestamp(device.nextchange_endperiod),
     ),
     FritzSensorEntityDescription(
         key="nextchange_preset",
         name="Next Scheduled Preset",
-        suitable=lambda device: device.has_thermostat
-        and device.nextchange_temperature is not None,
-        native_value=lambda device: PRESET_ECO
-        if device.nextchange_temperature == device.eco_temperature
-        else PRESET_COMFORT,
+        suitable=suitable_nextchange_temperature,
+        native_value=value_nextchange_preset,
     ),
     FritzSensorEntityDescription(
         key="scheduled_preset",
         name="Current Scheduled Preset",
-        suitable=lambda device: device.has_thermostat
-        and device.nextchange_temperature is not None,
-        native_value=lambda device: PRESET_COMFORT
-        if device.nextchange_temperature == device.eco_temperature
-        else PRESET_ECO,
+        suitable=suitable_nextchange_temperature,
+        native_value=value_scheduled_preset,
     ),
 )
 
@@ -186,14 +220,14 @@ async def async_setup_entry(
     async_add_entities(
         [
             FritzBoxSensor(coordinator, ain, description)
-            for ain, device in coordinator.data.items()
+            for ain, device in coordinator.data.devices.items()
             for description in SENSOR_TYPES
             if description.suitable(device)
         ]
     )
 
 
-class FritzBoxSensor(FritzBoxEntity, SensorEntity):
+class FritzBoxSensor(FritzBoxDeviceEntity, SensorEntity):
     """The entity class for FRITZ!SmartHome sensors."""
 
     entity_description: FritzSensorEntityDescription
@@ -201,4 +235,4 @@ class FritzBoxSensor(FritzBoxEntity, SensorEntity):
     @property
     def native_value(self) -> StateType | datetime:
         """Return the state of the sensor."""
-        return self.entity_description.native_value(self.device)
+        return self.entity_description.native_value(self.data)
