@@ -11,24 +11,14 @@ from typing import Any, Final, TypedDict, final
 
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import (
-    LENGTH_INCHES,
-    LENGTH_KILOMETERS,
-    LENGTH_MILES,
-    LENGTH_MILLIMETERS,
     PRECISION_HALVES,
     PRECISION_TENTHS,
     PRECISION_WHOLE,
-    PRESSURE_HPA,
-    PRESSURE_INHG,
-    PRESSURE_MBAR,
-    PRESSURE_MMHG,
-    SPEED_FEET_PER_SECOND,
-    SPEED_KILOMETERS_PER_HOUR,
-    SPEED_KNOTS,
-    SPEED_METERS_PER_SECOND,
-    SPEED_MILES_PER_HOUR,
-    TEMP_CELSIUS,
-    TEMP_FAHRENHEIT,
+    UnitOfLength,
+    UnitOfPrecipitationDepth,
+    UnitOfPressure,
+    UnitOfSpeed,
+    UnitOfTemperature,
 )
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers.config_validation import (  # noqa: F401
@@ -38,14 +28,13 @@ from homeassistant.helpers.config_validation import (  # noqa: F401
 from homeassistant.helpers.entity import Entity, EntityDescription
 from homeassistant.helpers.entity_component import EntityComponent
 from homeassistant.helpers.typing import ConfigType
-from homeassistant.util import (
-    distance as distance_util,
-    pressure as pressure_util,
-    speed as speed_util,
-    temperature as temperature_util,
+from homeassistant.util.unit_conversion import (
+    DistanceConverter,
+    PressureConverter,
+    SpeedConverter,
+    TemperatureConverter,
 )
-
-# mypy: allow-untyped-defs, no-check-untyped-defs
+from homeassistant.util.unit_system import US_CUSTOMARY_SYSTEM
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -101,41 +90,41 @@ SCAN_INTERVAL = timedelta(seconds=30)
 
 ROUNDING_PRECISION = 2
 
-VALID_UNITS_PRESSURE: tuple[str, ...] = (
-    PRESSURE_HPA,
-    PRESSURE_MBAR,
-    PRESSURE_INHG,
-    PRESSURE_MMHG,
-)
-VALID_UNITS_TEMPERATURE: tuple[str, ...] = (
-    TEMP_CELSIUS,
-    TEMP_FAHRENHEIT,
-)
-VALID_UNITS_PRECIPITATION: tuple[str, ...] = (
-    LENGTH_MILLIMETERS,
-    LENGTH_INCHES,
-)
-VALID_UNITS_VISIBILITY: tuple[str, ...] = (
-    LENGTH_KILOMETERS,
-    LENGTH_MILES,
-)
-VALID_UNITS_WIND_SPEED: tuple[str, ...] = (
-    SPEED_FEET_PER_SECOND,
-    SPEED_KILOMETERS_PER_HOUR,
-    SPEED_KNOTS,
-    SPEED_METERS_PER_SECOND,
-    SPEED_MILES_PER_HOUR,
-)
-
-UNIT_CONVERSIONS: dict[str, Callable[[float, str, str], float]] = {
-    ATTR_WEATHER_PRESSURE_UNIT: pressure_util.convert,
-    ATTR_WEATHER_TEMPERATURE_UNIT: temperature_util.convert,
-    ATTR_WEATHER_VISIBILITY_UNIT: distance_util.convert,
-    ATTR_WEATHER_PRECIPITATION_UNIT: distance_util.convert,
-    ATTR_WEATHER_WIND_SPEED_UNIT: speed_util.convert,
+VALID_UNITS_PRESSURE: set[str] = {
+    UnitOfPressure.HPA,
+    UnitOfPressure.MBAR,
+    UnitOfPressure.INHG,
+    UnitOfPressure.MMHG,
+}
+VALID_UNITS_TEMPERATURE: set[str] = {
+    UnitOfTemperature.CELSIUS,
+    UnitOfTemperature.FAHRENHEIT,
+}
+VALID_UNITS_PRECIPITATION: set[str] = {
+    UnitOfPrecipitationDepth.MILLIMETERS,
+    UnitOfPrecipitationDepth.INCHES,
+}
+VALID_UNITS_VISIBILITY: set[str] = {
+    UnitOfLength.KILOMETERS,
+    UnitOfLength.MILES,
+}
+VALID_UNITS_WIND_SPEED: set[str] = {
+    UnitOfSpeed.FEET_PER_SECOND,
+    UnitOfSpeed.KILOMETERS_PER_HOUR,
+    UnitOfSpeed.KNOTS,
+    UnitOfSpeed.METERS_PER_SECOND,
+    UnitOfSpeed.MILES_PER_HOUR,
 }
 
-VALID_UNITS: dict[str, tuple[str, ...]] = {
+UNIT_CONVERSIONS: dict[str, Callable[[float, str, str], float]] = {
+    ATTR_WEATHER_PRESSURE_UNIT: PressureConverter.convert,
+    ATTR_WEATHER_TEMPERATURE_UNIT: TemperatureConverter.convert,
+    ATTR_WEATHER_VISIBILITY_UNIT: DistanceConverter.convert,
+    ATTR_WEATHER_PRECIPITATION_UNIT: DistanceConverter.convert,
+    ATTR_WEATHER_WIND_SPEED_UNIT: SpeedConverter.convert,
+}
+
+VALID_UNITS: dict[str, set[str]] = {
     ATTR_WEATHER_PRESSURE_UNIT: VALID_UNITS_PRESSURE,
     ATTR_WEATHER_TEMPERATURE_UNIT: VALID_UNITS_TEMPERATURE,
     ATTR_WEATHER_VISIBILITY_UNIT: VALID_UNITS_VISIBILITY,
@@ -143,6 +132,8 @@ VALID_UNITS: dict[str, tuple[str, ...]] = {
     ATTR_WEATHER_WIND_SPEED_UNIT: VALID_UNITS_WIND_SPEED,
 }
 
+# mypy: disallow-any-generics
+
 
 def round_temperature(temperature: float | None, precision: float) -> float | None:
     """Convert temperature into preferred precision for display."""
@@ -185,7 +176,7 @@ class Forecast(TypedDict, total=False):
 
 async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
     """Set up the weather component."""
-    component = hass.data[DOMAIN] = EntityComponent(
+    component = hass.data[DOMAIN] = EntityComponent[WeatherEntity](
         _LOGGER, DOMAIN, hass, SCAN_INTERVAL
     )
     await component.async_setup(config)
@@ -194,13 +185,13 @@ async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
 
 async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Set up a config entry."""
-    component: EntityComponent = hass.data[DOMAIN]
+    component: EntityComponent[WeatherEntity] = hass.data[DOMAIN]
     return await component.async_setup_entry(entry)
 
 
 async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Unload a config entry."""
-    component: EntityComponent = hass.data[DOMAIN]
+    component: EntityComponent[WeatherEntity] = hass.data[DOMAIN]
     return await component.async_unload_entry(entry)
 
 
@@ -419,7 +410,9 @@ class WeatherEntity(Entity):
 
         Should not be set by integrations.
         """
-        return PRESSURE_HPA if self.hass.config.units.is_metric else PRESSURE_INHG
+        if self.hass.config.units is US_CUSTOMARY_SYSTEM:
+            return UnitOfPressure.INHG
+        return UnitOfPressure.HPA
 
     @final
     @property
@@ -481,11 +474,9 @@ class WeatherEntity(Entity):
 
         Should not be set by integrations.
         """
-        return (
-            SPEED_KILOMETERS_PER_HOUR
-            if self.hass.config.units.is_metric
-            else SPEED_MILES_PER_HOUR
-        )
+        if self.hass.config.units is US_CUSTOMARY_SYSTEM:
+            return UnitOfSpeed.MILES_PER_HOUR
+        return UnitOfSpeed.KILOMETERS_PER_HOUR
 
     @final
     @property
@@ -620,15 +611,15 @@ class WeatherEntity(Entity):
             return self._attr_precision
         return (
             PRECISION_TENTHS
-            if self._temperature_unit == TEMP_CELSIUS
+            if self._temperature_unit == UnitOfTemperature.CELSIUS
             else PRECISION_WHOLE
         )
 
     @final
     @property
-    def state_attributes(self):
+    def state_attributes(self) -> dict[str, Any]:
         """Return the state attributes, converted from native units to user-configured units."""
-        data = {}
+        data: dict[str, Any] = {}
 
         precision = self.precision
 
@@ -705,9 +696,9 @@ class WeatherEntity(Entity):
         data[ATTR_WEATHER_PRECIPITATION_UNIT] = self._precipitation_unit
 
         if self.forecast is not None:
-            forecast = []
-            for forecast_entry in self.forecast:
-                forecast_entry = dict(forecast_entry)
+            forecast: list[dict[str, Any]] = []
+            for existing_forecast_entry in self.forecast:
+                forecast_entry: dict[str, Any] = dict(existing_forecast_entry)
 
                 temperature = forecast_entry.pop(
                     ATTR_FORECAST_NATIVE_TEMP, forecast_entry.get(ATTR_FORECAST_TEMP)
