@@ -2,37 +2,41 @@
 from __future__ import annotations
 
 import asyncio
-from collections.abc import Coroutine
+import contextlib
 from contextlib import suppress
-import fnmatch
-import ipaddress
+from dataclasses import dataclass
+from fnmatch import translate
+from functools import lru_cache
+from ipaddress import IPv4Address, IPv6Address, ip_address
 import logging
+import re
 import socket
-from typing import Any, TypedDict, cast
+import sys
+from typing import Any, Final, cast
 
 import voluptuous as vol
-from zeroconf import (
-    InterfaceChoice,
-    IPVersion,
-    NonUniqueNameException,
-    ServiceStateChange,
-)
+from zeroconf import InterfaceChoice, IPVersion, ServiceStateChange
 from zeroconf.asyncio import AsyncServiceInfo
 
-from homeassistant import config_entries, util
+from homeassistant import config_entries
 from homeassistant.components import network
+from homeassistant.components.network import MDNS_TARGET_IP, async_get_source_ip
 from homeassistant.components.network.models import Adapter
-from homeassistant.const import (
-    EVENT_HOMEASSISTANT_START,
-    EVENT_HOMEASSISTANT_STARTED,
-    EVENT_HOMEASSISTANT_STOP,
-    __version__,
-)
+from homeassistant.const import EVENT_HOMEASSISTANT_STOP, __version__
 from homeassistant.core import Event, HomeAssistant, callback
-from homeassistant.data_entry_flow import FlowResult
+from homeassistant.data_entry_flow import BaseServiceInfo
+from homeassistant.helpers import discovery_flow, instance_id
 import homeassistant.helpers.config_validation as cv
 from homeassistant.helpers.network import NoURLAvailableError, get_url
-from homeassistant.loader import async_get_homekit, async_get_zeroconf, bind_hass
+from homeassistant.helpers.typing import ConfigType
+from homeassistant.loader import (
+    Integration,
+    async_get_homekit,
+    async_get_integration,
+    async_get_zeroconf,
+    bind_hass,
+)
+from homeassistant.setup import async_when_setup_or_start
 
 from .models import HaAsyncServiceBrowser, HaAsyncZeroconf, HaZeroconf
 from .usage import install_multiple_zeroconf_catcher
@@ -48,6 +52,10 @@ HOMEKIT_TYPES = [
     "_hap._udp.local.",
 ]
 
+# Top level keys we support matching against in properties that are always matched in
+# lower case. ex: ZeroconfServiceInfo.name
+LOWER_MATCH_ATTRS = {"name"}
+
 CONF_DEFAULT_INTERFACE = "default_interface"
 CONF_IPV6 = "ipv6"
 DEFAULT_DEFAULT_INTERFACE = True
@@ -56,8 +64,6 @@ DEFAULT_IPV6 = True
 HOMEKIT_PAIRED_STATUS_FLAG = "sf"
 HOMEKIT_MODEL = "md"
 
-MDNS_TARGET_IP = "224.0.0.251"
-
 # Property key=value has a max length of 255
 # so we use 230 to leave space for key=
 MAX_PROPERTY_VALUE_LEN = 230
@@ -65,6 +71,11 @@ MAX_PROPERTY_VALUE_LEN = 230
 # Dns label max length
 MAX_NAME_LEN = 63
 
+ATTR_PROPERTIES: Final = "properties"
+
+# Attributes for ZeroconfServiceInfo[ATTR_PROPERTIES]
+ATTR_PROPERTIES_ID: Final = "id"
+
 CONFIG_SCHEMA = vol.Schema(
     {
         DOMAIN: vol.All(
@@ -82,10 +93,12 @@ CONFIG_SCHEMA = vol.Schema(
 )
 
 
-class HaServiceInfo(TypedDict):
+@dataclass
+class ZeroconfServiceInfo(BaseServiceInfo):
     """Prepared info from mDNS entries."""
 
     host: str
+    addresses: list[str]
     port: int | None
     hostname: str
     type: str
@@ -93,14 +106,6 @@ class HaServiceInfo(TypedDict):
     properties: dict[str, Any]
 
 
-class ZeroconfFlow(TypedDict):
-    """A queued zeroconf discovery flow."""
-
-    domain: str
-    context: dict[str, Any]
-    data: HaServiceInfo
-
-
 @bind_hass
 async def async_get_instance(hass: HomeAssistant) -> HaZeroconf:
     """Zeroconf instance to be shared with other integrations that use it."""
@@ -134,34 +139,49 @@ async def _async_get_instance(hass: HomeAssistant, **zcargs: Any) -> HaAsyncZero
     return aio_zc
 
 
-def _async_use_default_interface(adapters: list[Adapter]) -> bool:
-    for adapter in adapters:
-        if adapter["enabled"] and not adapter["default"]:
-            return False
-    return True
+@callback
+def _async_zc_has_functional_dual_stack() -> bool:
+    """Return true for platforms that not support IP_ADD_MEMBERSHIP on an AF_INET6 socket.
+
+    Zeroconf only supports a single listen socket at this time.
+    """
+    return not sys.platform.startswith("freebsd") and not sys.platform.startswith(
+        "darwin"
+    )
 
 
-async def async_setup(hass: HomeAssistant, config: dict) -> bool:
+async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
     """Set up Zeroconf and make Home Assistant discoverable."""
-    zc_args: dict = {}
+    zc_args: dict = {"ip_version": IPVersion.V4Only}
 
     adapters = await network.async_get_adapters(hass)
-    if _async_use_default_interface(adapters):
+
+    ipv6 = False
+    if _async_zc_has_functional_dual_stack():
+        if any(adapter["enabled"] and adapter["ipv6"] for adapter in adapters):
+            ipv6 = True
+            zc_args["ip_version"] = IPVersion.All
+    elif not any(adapter["enabled"] and adapter["ipv4"] for adapter in adapters):
+        zc_args["ip_version"] = IPVersion.V6Only
+        ipv6 = True
+
+    if not ipv6 and network.async_only_default_interface_enabled(adapters):
         zc_args["interfaces"] = InterfaceChoice.Default
     else:
-        interfaces = zc_args["interfaces"] = []
-        for adapter in adapters:
-            if not adapter["enabled"]:
-                continue
-            if ipv4s := adapter["ipv4"]:
-                interfaces.append(ipv4s[0]["address"])
-            elif ipv6s := adapter["ipv6"]:
-                interfaces.append(ipv6s[0]["scope_id"])
-
-    ipv6 = True
-    if not any(adapter["enabled"] and adapter["ipv6"] for adapter in adapters):
-        ipv6 = False
-        zc_args["ip_version"] = IPVersion.V4Only
+        zc_args["interfaces"] = [
+            str(source_ip)
+            for source_ip in await network.async_get_enabled_source_ips(hass)
+            if not source_ip.is_loopback
+            and not (isinstance(source_ip, IPv6Address) and source_ip.is_global)
+            and not (
+                isinstance(source_ip, IPv6Address)
+                and zc_args["ip_version"] == IPVersion.V4Only
+            )
+            and not (
+                isinstance(source_ip, IPv4Address)
+                and zc_args["ip_version"] == IPVersion.V6Only
+            )
+        ]
 
     aio_zc = await _async_get_instance(hass, **zc_args)
     zeroconf = cast(HaZeroconf, aio_zc.zeroconf)
@@ -171,34 +191,64 @@ async def async_setup(hass: HomeAssistant, config: dict) -> bool:
     discovery = ZeroconfDiscovery(hass, zeroconf, zeroconf_types, homekit_models, ipv6)
     await discovery.async_setup()
 
-    async def _async_zeroconf_hass_start(_event: Event) -> None:
+    async def _async_zeroconf_hass_start(hass: HomeAssistant, comp: str) -> None:
         """Expose Home Assistant on zeroconf when it starts.
 
         Wait till started or otherwise HTTP is not up and running.
         """
-        uuid = await hass.helpers.instance_id.async_get()
+        uuid = await instance_id.async_get(hass)
         await _async_register_hass_zc_service(hass, aio_zc, uuid)
 
-    @callback
-    def _async_start_discovery(_event: Event) -> None:
-        """Start processing flows."""
-        discovery.async_start()
-
     async def _async_zeroconf_hass_stop(_event: Event) -> None:
         await discovery.async_stop()
 
     hass.bus.async_listen_once(EVENT_HOMEASSISTANT_STOP, _async_zeroconf_hass_stop)
-    hass.bus.async_listen_once(EVENT_HOMEASSISTANT_START, _async_zeroconf_hass_start)
-    hass.bus.async_listen_once(EVENT_HOMEASSISTANT_STARTED, _async_start_discovery)
+    async_when_setup_or_start(hass, "frontend", _async_zeroconf_hass_start)
 
     return True
 
 
+def _get_announced_addresses(
+    adapters: list[Adapter],
+    first_ip: bytes | None = None,
+) -> list[bytes]:
+    """Return a list of IP addresses to announce via zeroconf.
+
+    If first_ip is not None, it will be the first address in the list.
+    """
+    addresses = {
+        addr.packed
+        for addr in [
+            ip_address(ip["address"])
+            for adapter in adapters
+            if adapter["enabled"]
+            for ip in cast(list, adapter["ipv6"]) + cast(list, adapter["ipv4"])
+        ]
+        if not (addr.is_unspecified or addr.is_loopback)
+    }
+    if first_ip:
+        address_list = [first_ip]
+        address_list.extend(addresses - set({first_ip}))
+    else:
+        address_list = list(addresses)
+    return address_list
+
+
+def _filter_disallowed_characters(name: str) -> str:
+    """Filter disallowed characters from a string.
+
+    . is a reversed character for zeroconf.
+    """
+    return name.replace(".", " ")
+
+
 async def _async_register_hass_zc_service(
     hass: HomeAssistant, aio_zc: HaAsyncZeroconf, uuid: str
 ) -> None:
     # Get instance UUID
-    valid_location_name = _truncate_location_name_to_valid(hass.config.location_name)
+    valid_location_name = _truncate_location_name_to_valid(
+        _filter_disallowed_characters(hass.config.location_name or "Home")
+    )
 
     params = {
         "location_name": valid_location_name,
@@ -222,12 +272,15 @@ async def _async_register_hass_zc_service(
     # Set old base URL based on external or internal
     params["base_url"] = params["external_url"] or params["internal_url"]
 
-    host_ip = util.get_local_ip()
+    adapters = await network.async_get_adapters(hass)
 
-    try:
+    # Puts the default IPv4 address first in the list to preserve compatibility,
+    # because some mDNS implementations ignores anything but the first announced address.
+    host_ip = await async_get_source_ip(hass, target_ip=MDNS_TARGET_IP)
+    host_ip_pton = None
+    if host_ip:
         host_ip_pton = socket.inet_pton(socket.AF_INET, host_ip)
-    except OSError:
-        host_ip_pton = socket.inet_pton(socket.AF_INET6, host_ip)
+    address_list = _get_announced_addresses(adapters, host_ip_pton)
 
     _suppress_invalid_properties(params)
 
@@ -235,52 +288,50 @@ async def _async_register_hass_zc_service(
         ZEROCONF_TYPE,
         name=f"{valid_location_name}.{ZEROCONF_TYPE}",
         server=f"{uuid}.local.",
-        addresses=[host_ip_pton],
+        addresses=address_list,
         port=hass.http.server_port,
         properties=params,
     )
 
     _LOGGER.info("Starting Zeroconf broadcast")
-    try:
-        await aio_zc.async_register_service(info)
-    except NonUniqueNameException:
-        _LOGGER.error(
-            "Home Assistant instance with identical name present in the local network"
-        )
+    await aio_zc.async_register_service(info, allow_name_change=True)
 
 
-class FlowDispatcher:
-    """Dispatch discovery flows."""
+def _match_against_data(
+    matcher: dict[str, str | dict[str, str]], match_data: dict[str, str]
+) -> bool:
+    """Check a matcher to ensure all values in match_data match."""
+    for key in LOWER_MATCH_ATTRS:
+        if key not in matcher:
+            continue
+        if key not in match_data:
+            return False
+        match_val = matcher[key]
+        assert isinstance(match_val, str)
 
-    def __init__(self, hass: HomeAssistant) -> None:
-        """Init the discovery dispatcher."""
-        self.hass = hass
-        self.pending_flows: list[ZeroconfFlow] = []
-        self.started = False
+        if not _memorized_fnmatch(match_data[key], match_val):
+            return False
+    return True
 
-    @callback
-    def async_start(self) -> None:
-        """Start processing pending flows."""
-        self.started = True
-        self.hass.loop.call_soon(self._async_process_pending_flows)
 
-    def _async_process_pending_flows(self) -> None:
-        for flow in self.pending_flows:
-            self.hass.async_create_task(self._init_flow(flow))
-        self.pending_flows = []
+def _match_against_props(matcher: dict[str, str], props: dict[str, str]) -> bool:
+    """Check a matcher to ensure all values in props."""
+    return not any(
+        key
+        for key in matcher
+        if key not in props or not _memorized_fnmatch(props[key].lower(), matcher[key])
+    )
 
-    def async_create(self, flow: ZeroconfFlow) -> None:
-        """Create and add or queue a flow."""
-        if self.started:
-            self.hass.async_create_task(self._init_flow(flow))
-        else:
-            self.pending_flows.append(flow)
 
-    def _init_flow(self, flow: ZeroconfFlow) -> Coroutine[None, None, FlowResult]:
-        """Create a flow."""
-        return self.hass.config_entries.flow.async_init(
-            flow["domain"], context=flow["context"], data=flow["data"]
-        )
+def is_homekit_paired(props: dict[str, Any]) -> bool:
+    """Check properties to see if a device is homekit paired."""
+    if HOMEKIT_PAIRED_STATUS_FLAG not in props:
+        return False
+    with contextlib.suppress(ValueError):
+        # 0 means paired and not discoverable by iOS clients)
+        return int(props[HOMEKIT_PAIRED_STATUS_FLAG]) == 0
+    # If we cannot tell, we assume its not paired
+    return False
 
 
 class ZeroconfDiscovery:
@@ -290,7 +341,7 @@ class ZeroconfDiscovery:
         self,
         hass: HomeAssistant,
         zeroconf: HaZeroconf,
-        zeroconf_types: dict[str, list[dict[str, str]]],
+        zeroconf_types: dict[str, list[dict[str, str | dict[str, str]]]],
         homekit_models: dict[str, str],
         ipv6: bool,
     ) -> None:
@@ -301,12 +352,10 @@ class ZeroconfDiscovery:
         self.homekit_models = homekit_models
         self.ipv6 = ipv6
 
-        self.flow_dispatcher: FlowDispatcher | None = None
         self.async_service_browser: HaAsyncServiceBrowser | None = None
 
     async def async_setup(self) -> None:
         """Start discovery."""
-        self.flow_dispatcher = FlowDispatcher(self.hass)
         types = list(self.zeroconf_types)
         # We want to make sure we know about other HomeAssistant
         # instances as soon as possible to avoid name conflicts
@@ -324,12 +373,6 @@ class ZeroconfDiscovery:
         if self.async_service_browser:
             await self.async_service_browser.async_cancel()
 
-    @callback
-    def async_start(self) -> None:
-        """Start processing discovery flows."""
-        assert self.flow_dispatcher is not None
-        self.flow_dispatcher.async_start()
-
     @callback
     def async_service_update(
         self,
@@ -365,12 +408,16 @@ class ZeroconfDiscovery:
             return
 
         _LOGGER.debug("Discovered new device %s %s", name, info)
-        assert self.flow_dispatcher is not None
+        props: dict[str, str] = info.properties
+        domain = None
 
         # If we can handle it as a HomeKit discovery, we do that here.
-        if service_type in HOMEKIT_TYPES:
-            if pending_flow := handle_homekit(self.hass, self.homekit_models, info):
-                self.flow_dispatcher.async_create(pending_flow)
+        if service_type in HOMEKIT_TYPES and (
+            domain := async_get_homekit_discovery_domain(self.homekit_models, props)
+        ):
+            discovery_flow.async_create_flow(
+                self.hass, domain, {"source": config_entries.SOURCE_HOMEKIT}, info
+            )
             # Continue on here as homekit_controller
             # still needs to get updates on devices
             # so it can see when the 'c#' field is updated.
@@ -378,73 +425,68 @@ class ZeroconfDiscovery:
             # We only send updates to homekit_controller
             # if the device is already paired in order to avoid
             # offering a second discovery for the same device
-            if pending_flow and HOMEKIT_PAIRED_STATUS_FLAG in info["properties"]:
-                try:
-                    # 0 means paired and not discoverable by iOS clients)
-                    if int(info["properties"][HOMEKIT_PAIRED_STATUS_FLAG]):
-                        return
-                except ValueError:
-                    # HomeKit pairing status unknown
-                    # likely bad homekit data
+            if not is_homekit_paired(props):
+                integration: Integration = await async_get_integration(
+                    self.hass, domain
+                )
+                # Since we prefer local control, if the integration that is being discovered
+                # is cloud AND the homekit device is UNPAIRED we still want to discovery it.
+                #
+                # Additionally if the integration is polling, HKC offers a local push
+                # experience for the user to control the device so we want to offer that
+                # as well.
+                #
+                # As soon as the device becomes paired, the config flow will be dismissed
+                # in the event the user does not want to pair with Home Assistant.
+                #
+                if not integration.iot_class or (
+                    not integration.iot_class.startswith("cloud")
+                    and "polling" not in integration.iot_class
+                ):
                     return
 
-        if "name" in info:
-            lowercase_name: str | None = info["name"].lower()
-        else:
-            lowercase_name = None
-
-        if "macaddress" in info["properties"]:
-            uppercase_mac: str | None = info["properties"]["macaddress"].upper()
-        else:
-            uppercase_mac = None
-
-        if "manufacturer" in info["properties"]:
-            lowercase_manufacturer: str | None = info["properties"][
-                "manufacturer"
-            ].lower()
-        else:
-            lowercase_manufacturer = None
+        match_data: dict[str, str] = {}
+        for key in LOWER_MATCH_ATTRS:
+            attr_value: str = getattr(info, key)
+            match_data[key] = attr_value.lower()
 
         # Not all homekit types are currently used for discovery
         # so not all service type exist in zeroconf_types
         for matcher in self.zeroconf_types.get(service_type, []):
             if len(matcher) > 1:
-                if "macaddress" in matcher and (
-                    uppercase_mac is None
-                    or not fnmatch.fnmatch(uppercase_mac, matcher["macaddress"])
-                ):
-                    continue
-                if "name" in matcher and (
-                    lowercase_name is None
-                    or not fnmatch.fnmatch(lowercase_name, matcher["name"])
-                ):
-                    continue
-                if "manufacturer" in matcher and (
-                    lowercase_manufacturer is None
-                    or not fnmatch.fnmatch(
-                        lowercase_manufacturer, matcher["manufacturer"]
-                    )
-                ):
+                if not _match_against_data(matcher, match_data):
                     continue
+                if ATTR_PROPERTIES in matcher:
+                    matcher_props = matcher[ATTR_PROPERTIES]
+                    assert isinstance(matcher_props, dict)
+                    if not _match_against_props(matcher_props, props):
+                        continue
 
-            flow: ZeroconfFlow = {
-                "domain": matcher["domain"],
-                "context": {"source": config_entries.SOURCE_ZEROCONF},
-                "data": info,
+            matcher_domain = matcher["domain"]
+            assert isinstance(matcher_domain, str)
+            context = {
+                "source": config_entries.SOURCE_ZEROCONF,
             }
-            self.flow_dispatcher.async_create(flow)
+            if domain:
+                # Domain of integration that offers alternative API to handle this device.
+                context["alternative_domain"] = domain
+
+            discovery_flow.async_create_flow(
+                self.hass,
+                matcher_domain,
+                context,
+                info,
+            )
 
 
-def handle_homekit(
-    hass: HomeAssistant, homekit_models: dict[str, str], info: HaServiceInfo
-) -> ZeroconfFlow | None:
+def async_get_homekit_discovery_domain(
+    homekit_models: dict[str, str], props: dict[str, Any]
+) -> str | None:
     """Handle a HomeKit discovery.
 
-    Return if discovery was forwarded.
+    Return the domain to forward the discovery data to
     """
     model = None
-    props = info["properties"]
-
     for key in props:
         if key.lower() == HOMEKIT_MODEL:
             model = props[key]
@@ -457,20 +499,16 @@ def handle_homekit(
         if (
             model != test_model
             and not model.startswith((f"{test_model} ", f"{test_model}-"))
-            and not fnmatch.fnmatch(model, test_model)
+            and not _memorized_fnmatch(model, test_model)
         ):
             continue
 
-        return {
-            "domain": homekit_models[test_model],
-            "context": {"source": config_entries.SOURCE_HOMEKIT},
-            "data": info,
-        }
+        return homekit_models[test_model]
 
     return None
 
 
-def info_from_service(service: AsyncServiceInfo) -> HaServiceInfo | None:
+def info_from_service(service: AsyncServiceInfo) -> ZeroconfServiceInfo | None:
     """Return prepared info from mDNS entries."""
     properties: dict[str, Any] = {"_raw": {}}
 
@@ -492,19 +530,44 @@ def info_from_service(service: AsyncServiceInfo) -> HaServiceInfo | None:
             if isinstance(value, bytes):
                 properties[key] = value.decode("utf-8")
 
-    if not service.addresses:
+    if not (addresses := service.addresses or service.parsed_addresses()):
+        return None
+    if (host := _first_non_link_local_address(addresses)) is None:
         return None
 
-    address = service.addresses[0]
+    return ZeroconfServiceInfo(
+        host=str(host),
+        addresses=service.parsed_addresses(),
+        port=service.port,
+        hostname=service.server,
+        type=service.type,
+        name=service.name,
+        properties=properties,
+    )
 
-    return {
-        "host": str(ipaddress.ip_address(address)),
-        "port": service.port,
-        "hostname": service.server,
-        "type": service.type,
-        "name": service.name,
-        "properties": properties,
-    }
+
+def _first_non_link_local_address(
+    addresses: list[bytes] | list[str],
+) -> str | None:
+    """Return the first ipv6 or non-link local ipv4 address, preferring IPv4."""
+    for address in addresses:
+        ip_addr = ip_address(address)
+        if (
+            not ip_addr.is_link_local
+            and not ip_addr.is_unspecified
+            and ip_addr.version == 4
+        ):
+            return str(ip_addr)
+    # If we didn't find a good IPv4 address, check for IPv6 addresses.
+    for address in addresses:
+        ip_addr = ip_address(address)
+        if (
+            not ip_addr.is_link_local
+            and not ip_addr.is_unspecified
+            and ip_addr.version == 6
+        ):
+            return str(ip_addr)
+    return None
 
 
 def _suppress_invalid_properties(properties: dict) -> None:
@@ -535,3 +598,24 @@ def _truncate_location_name_to_valid(location_name: str) -> str:
         location_name,
     )
     return location_name.encode("utf-8")[:MAX_NAME_LEN].decode("utf-8", "ignore")
+
+
+@lru_cache(maxsize=4096, typed=True)
+def _compile_fnmatch(pattern: str) -> re.Pattern:
+    """Compile a fnmatch pattern."""
+    return re.compile(translate(pattern))
+
+
+@lru_cache(maxsize=1024, typed=True)
+def _memorized_fnmatch(name: str, pattern: str) -> bool:
+    """Memorized version of fnmatch that has a larger lru_cache.
+
+    The default version of fnmatch only has a lru_cache of 256 entries.
+    With many devices we quickly reach that limit and end up compiling
+    the same pattern over and over again.
+
+    Zeroconf has its own memorized fnmatch with its own lru_cache
+    since the data is going to be relatively the same
+    since the devices will not change frequently
+    """
+    return bool(_compile_fnmatch(pattern).match(name))
